"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing_extensions import TypedDict


class CustomMappingsTypedDict(TypedDict):
    r"""When custom mappings are configured on the resource, the result is included here."""


class CustomMappings(BaseModel):
    r"""When custom mappings are configured on the resource, the result is included here."""
