"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.apideck_companies import ApideckCompanies
from apideck_unify.apideck_departments import ApideckDepartments
from apideck_unify.employeepayrolls import EmployeePayrolls
from apideck_unify.employees import Employees
from apideck_unify.employeeschedules_sdk import EmployeeSchedulesSDK
from apideck_unify.payrolls import Payrolls
from apideck_unify.timeoffrequests import TimeOffRequests


class Hris(BaseSDK):
    employees: Employees
    companies: ApideckCompanies
    departments: ApideckDepartments
    payrolls: Payrolls
    employee_payrolls: EmployeePayrolls
    employee_schedules: EmployeeSchedulesSDK
    time_off_requests: TimeOffRequests

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.employees = Employees(self.sdk_configuration)
        self.companies = ApideckCompanies(self.sdk_configuration)
        self.departments = ApideckDepartments(self.sdk_configuration)
        self.payrolls = Payrolls(self.sdk_configuration)
        self.employee_payrolls = EmployeePayrolls(self.sdk_configuration)
        self.employee_schedules = EmployeeSchedulesSDK(self.sdk_configuration)
        self.time_off_requests = TimeOffRequests(self.sdk_configuration)
