"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CreateCallbackStateTypedDict(TypedDict):
    redirect_uri: NotRequired[str]
    r"""The redirect URI to be used after the connection is created."""


class CreateCallbackState(BaseModel):
    redirect_uri: Optional[str] = None
    r"""The redirect URI to be used after the connection is created."""
