"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.apideck_customers import ApideckCustomers
from apideck_unify.orders import Orders
from apideck_unify.products import Products
from apideck_unify.stores import Stores


class Ecommerce(BaseSDK):
    orders: Orders
    products: Products
    customers: ApideckCustomers
    stores: Stores

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.orders = Orders(self.sdk_configuration)
        self.products = Products(self.sdk_configuration)
        self.customers = ApideckCustomers(self.sdk_configuration)
        self.stores = Stores(self.sdk_configuration)
