"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from apideck_unify.utils import FieldMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhookEventLogsFilterServiceTypedDict(TypedDict):
    id: NotRequired[str]


class WebhookEventLogsFilterService(BaseModel):
    id: Annotated[Optional[str], FieldMetadata(query=True)] = None


class WebhookEventLogsFilterTypedDict(TypedDict):
    exclude_apis: NotRequired[Nullable[str]]
    service: NotRequired[Nullable[WebhookEventLogsFilterServiceTypedDict]]
    consumer_id: NotRequired[Nullable[str]]
    entity_type: NotRequired[Nullable[str]]
    event_type: NotRequired[Nullable[str]]


class WebhookEventLogsFilter(BaseModel):
    exclude_apis: Annotated[OptionalNullable[str], FieldMetadata(query=True)] = UNSET

    service: Annotated[
        OptionalNullable[WebhookEventLogsFilterService], FieldMetadata(query=True)
    ] = UNSET

    consumer_id: Annotated[OptionalNullable[str], FieldMetadata(query=True)] = UNSET

    entity_type: Annotated[OptionalNullable[str], FieldMetadata(query=True)] = UNSET

    event_type: Annotated[OptionalNullable[str], FieldMetadata(query=True)] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "exclude_apis",
            "service",
            "consumer_id",
            "entity_type",
            "event_type",
        ]
        nullable_fields = [
            "exclude_apis",
            "service",
            "consumer_id",
            "entity_type",
            "event_type",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
