"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class FileType(str, Enum):
    r"""The type of resource. Could be file, folder or url"""

    FILE = "file"
    FOLDER = "folder"
    URL = "url"
