"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class EmploymentStatus(str, Enum):
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""

    ACTIVE = "active"
    INACTIVE = "inactive"
    TERMINATED = "terminated"
    OTHER = "other"
