"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PaymentUnit(str, Enum):
    r"""Unit of measurement for employee compensation."""

    HOUR = "hour"
    WEEK = "week"
    MONTH = "month"
    YEAR = "year"
    PAYCHECK = "paycheck"
    OTHER = "other"
